package models

import (
	"eta/eta_mini_bridge/utils"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type Classify struct {
	Id           int       `description:"分类id"`
	ClassifyName string    `description:"分类名称"`
	Sort         int       `json:"-"`
	ParentId     int       `description:"父级分类id"`
	HasChild     int       `description:"是否有子分类"`
	CreateTime   time.Time `description:"创建时间"`
	ModifyTime   time.Time `description:"修改时间"`
	Level        int       `description:"分类层级"`
}

type ClassifyView struct {
	Id           int             `description:"分类id"`
	ClassifyName string          `description:"分类名称"`
	Sort         int             `json:"-"`
	ParentId     int             `description:"父级分类id"`
	CreateTime   time.Time       `description:"创建时间"`
	ModifyTime   time.Time       `description:"修改时间"`
	Level        int             `description:"分类层级"`
	HasChild     int             `description:"是否有子分类0：下面没有子分类，1：下面有子分类" json:"-"`
	Child        []*ClassifyView `description:"子分类"`
}

type ClassifyDetail struct {
	ClassifyId     int       `description:"分类id"`
	ClassifyName   string    `description:"分类名称"`
	Sort           int       `json:"-"`
	ParentId       int       `description:"父级分类id"`
	CreateTime     time.Time `description:"创建时间"`
	ModifyTime     time.Time `description:"修改时间"`
	Abstract       string    `description:"栏目简介"`
	Descript       string    `description:"分享描述"`
	ReportAuthor   string    `description:"栏目作者"`
	AuthorDescript string    `description:"作者简介"`
	ColumnImgUrl   string    `description:"栏目配图"`
	HeadImgUrl     string    `description:"头部banner"`
	AvatarImgUrl   string    `description:"头像"`
	ReportImgUrl   string    `description:"报告配图"`
	HomeImgUrl     string    `description:"首页配图"`
	Stage          int       `description:"最新期数"`
}

func GetClassifyListByIds(ids []int) (items []*ClassifyView, err error) {
	if len(ids) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM classify WHERE enabled=1 AND id IN (` + utils.GetOrmReplaceHolder(len(ids)) + `)`
	_, err = o.Raw(sql, ids).QueryRows(&items)
	return
}

func GetChildClassifyListByIds(ids []int) (items []*ClassifyView, err error) {
	if len(ids) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM classify WHERE enabled=1 AND parent_id IN (` + utils.GetOrmReplaceHolder(len(ids)) + `)`
	_, err = o.Raw(sql, ids).QueryRows(&items)
	return
}

func GetClassifyById(id int) (item *ClassifyView, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM classify WHERE id = ?`
	err = o.Raw(sql, id).QueryRow(&item)
	return
}

func GetClassifyList() (items []*ClassifyView, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM classify `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

func GetClassifyEnabledList() (items []*ClassifyView, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := `SELECT * FROM classify WHERE enabled=1 `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}
